 
import 'dart:io';

import 'package:origami_redis/models/redisRoute.dart';
import 'package:origami_telegram/models/telegramConfig.dart';
import 'package:origami_telegram/models/telegramResponseModel.dart'; 
import 'package:origamicore/models/packageIndex.dart';
import 'package:origamicore/models/serviceRoute.dart';
import 'package:origamicore/routing/router.dart';
import 'package:teledart/model.dart';
import 'package:teledart/teledart.dart';
import 'package:teledart/telegram.dart';
import 'dart:io' as io;

import 'services/stateMachine/stateMachine.dart';

class TelegramIndex extends PackageIndex
{ 
  TelegramConfig _cf=TelegramConfig(apiKey: '');
  StateMachine _stateMachine ;
  TeleDart _teledart;
  @override
  Future jsonConfig(config) async { 
    Router.addRoute('endpoint', ServiceRoute(services: {
      
    }));
    for(var a in config)
    {
      if(a['type']=='telegram')
      {
        _cf=TelegramConfig.fromJson(a);
      }
    } 
  } 
  @override
  void restart() {
    // TODO: implement restart
  }

  @override
  Future start() async { 

    _stateMachine= StateMachine();
    await _stateMachine.fromFile(Directory.current.path+ (_cf.stateMachinePath??''));
    var teledart = TeleDart(Telegram(_cf.apiKey), Event());
    _teledart=teledart;
    await teledart.start().then((me) => print('${me.username} is initialised'));

    teledart.onCallbackQuery().listen((message) async{
      var text = message.data;

      var state=await getState(message.message.chat.id);
      
      var session={
        'userid':message.message.chat.id,
      };
      var request = await _stateMachine.findServiceRequest(state,text,session);
      if(request.state.loading!=null)
      {
        await teledart.telegram.editMessageText(request.state.loading??'',message_id: message.message.message_id,chat_id: message.message.chat.id);
      }
      var resp = await _stateMachine.findTextRoute(state,text,session);
      await setState(resp.state,message.message.chat.id) ;
      await sendMessage(message.message.chat,resp);
      try{
        await teledart.telegram.deleteMessage(message.message.chat.id , message.message.message_id);

      }catch(exp){}
     });
    teledart.onCommand().listen((message)async {
      await setState('',message.chat.id) ;
      var text = message.text;
      var session={
        'userid':message.chat.id
      };
      var resp = await _stateMachine.runCommand(text,session);
      if(resp.state.isNotEmpty)
      {
        await setState(resp.state,message.chat.id) ;
      }
      await sendMessage(message.chat,resp);
    });
    teledart
        .onMessage()
        .listen((message)async{ 
            var text=message.text; 
            if(message.contact!=null)
            {
              
                text = '/contact';
                var session={
                  'userid':message.chat.id,
                  'contact':message.contact.toJson()
                };
                var resp =await _stateMachine.runCommand(text,session);
                await sendMessage(message.chat,resp);
                return;
            } 
            var state=await getState(message.chat.id); 
            var session={
              'userid':message.chat.id,
            };
            if(text==null)
            {
              return;
            }
            var request = await _stateMachine.findServiceRequest(state,text,session);
            if(request.state.deletable )
            {
              try{
                await teledart.telegram.deleteMessage(message.chat.id,message.message_id );
              }catch(exp){ 
              }
              // await teledart.telegram.editMessageText(request.state.loading,
              // message_id: message.message_id,chat_id: message.chat.id);
            }
            var resp = await _stateMachine.findTextRoute(state,text,session);
            await setState(resp.state,message.chat.id) ;
            await sendMessage(message.chat,resp);
            return;
            //await message.reply('Stand with Hong Kong');
           
           });
  }

  @override
  void stop() {
    // TODO: implement stop
  }
  
  Future sendMessage(Chat chat,TelegramResponseModel response) async
  { 
    var keyboard=<List<KeyboardButton>>[]; 
    var inlinekeyboard=<List<InlineKeyboardButton>>[]; 
      
    for(var a in response.button)
    {
      var row=<KeyboardButton>[];
      var inlinerow=<InlineKeyboardButton>[];
      for(var b in a)
      {
          if(b.url.isNotEmpty)
          {
            inlinerow.add(InlineKeyboardButton(text: b.title ,url: b.url));
          }
          else
          {
            row.add(KeyboardButton(text:b.title,request_contact: b.showMobile, ));
            inlinerow.add(InlineKeyboardButton(text: b.title,callback_data: b.action ,));
          }
      } 
      keyboard.add(row);
      inlinekeyboard.add(inlinerow);
    } 
    ReplyMarkup markup=ReplyKeyboardMarkup(keyboard: keyboard,one_time_keyboard: true,selective: true);  
    if(response.inlineKeyboard)
    {
      markup=InlineKeyboardMarkup(inline_keyboard: inlinekeyboard);
    }
    if(response.type==TelegramResponseType.Text )
    { 
      if(response.text!='')
      {
        await _teledart.telegram.sendMessage(chat.id, response.text,reply_markup:markup,parse_mode: response.parseMode??'' );

      }
    }
    if(response.type==TelegramResponseType.Image )
    {
      await _teledart.telegram.sendPhoto(chat.id, io.File(response.photo),caption: response.text,reply_markup:markup,parse_mode: response.parseMode??'' );
    }
     

  }
  Future setState(String state,int userid)async
  {
    await RedisRoute().setValue(_cf.rcontext, 'TelegramState'+userid.toString(),state);
  }
  Future<String> getState(int userid)async
  {
    return await RedisRoute().getStringValue(_cf.rcontext, 'TelegramState'+userid.toString());
  }
}